# frozen_string_literal: true

require 'spec_helper'

module Roqua::CsvExport
  describe MetaData, needs_db: true do
    let(:dossier) { Models::Dossier.create external_identifier: 'doss1' }
    let(:response) { Models::Response.new completed_at: Time.local(2018, 11, 1), dossier: dossier }
    let(:professional) { Models::Professional.create external_identifier: 'prof1'}
    let(:respondent) { Models::Respondent.create type: 'parent', label: 'Stiefvader' }

    subject { described_class.send(attribute, response) }

    describe '#anonymous' do
      let(:attribute) { :anonymous }

      it 'returns empty string if response is not anonymous' do
        expect(subject).to eq ''
      end

      it 'returns "anonymous" if response is anonymous' do
        response.anonymous = true
        expect(subject).to eq 'anonymous'
      end
    end

    describe '#compl_by' do
      let(:attribute) { :compl_by }

      it 'returns the external_identifier of the entered_by' do
        response.entered_by = professional
        expect(subject).to eq 'prof1'
      end

      it 'returns the external_identifier of the dossier if entered_by is not present' do
        expect(subject).to eq 'doss1'
      end

      it 'returns nil if the response is anonymous from dossier' do
        response.anonymous = true
        expect(subject).to eq nil
      end

      it 'returns entered_by if the response is anonymous from profess' do
        response.anonymous = true
        response.entered_by = professional
        expect(subject).to eq 'prof1'
      end

      it 'returns nil if the response was not completed' do
        response.completed_at = nil
        expect(subject).to eq nil
      end
    end

    describe '#completed_at' do
      let(:attribute) { :completed_at }

      it 'returns the completed_at' do
        expect(subject).to eq Time.local(2018, 11, 1)
      end
    end

    describe '#date' do
      let(:attribute) { :date }

      it 'returns the observation_time' do
        response.observation_time = Time.local(2018, 11, 2)
        expect(subject).to eq Time.local(2018, 11, 2)
      end
    end

    describe '#id' do
      let(:attribute) { :id }

      it 'returns the id' do
        response.save
        expect(subject).to eq response.id
      end
    end

    describe '#location' do
      let(:attribute) { :location }
      let(:response) { Models::Response.exclude(team_id: nil).first }

      it 'returns the team name' do
        expect(subject).to eq 'Team 1'
      end
    end

    describe '#measurement' do
      let(:attribute) { :measurement }
      let(:response) { Models::Response.exclude(measurement_id: nil).first }

      it 'returns the measurement name' do
        expect(subject).to eq 'Measurement 1'
      end
    end

    describe '#non_reponse' do
      let(:attribute) { :non_response }
      let(:response) { Models::Response.exclude(non_response_id: nil).first }

      it 'returns the non_response id' do
        expect(subject).to be_present
        expect(subject).to eq response.non_response_id
      end
    end

    describe '#notes' do
      let(:attribute) { :notes }

      it 'returns the non_response id' do
        response.notes = "Remember"
        expect(subject).to eq 'Remember'
      end
    end

    describe '#open_from' do
      let(:attribute) { :open_from }

      it 'returns the non_response id' do
        response.open_from = Time.local(2018, 11, 3)
        expect(subject).to eq Time.local(2018, 11, 3)
      end
    end

    describe '#patient_id' do
      let(:attribute) { :patient_id }

      it 'returns the external_identifier of the dossier' do
        expect(subject).to eq 'doss1'
      end

      it 'returns nil if the response is anonymous' do
        response.anonymous = true
        expect(subject).to eq nil
      end
    end

    describe '#protocol' do
      let(:attribute) { :protocol }
      let(:response) { Models::Response.exclude(measurement_id: nil).first }

      it 'returns the protocol name' do
        expect(subject).to eq 'Protocol 1'
      end
    end

    describe '#respondent_id' do
      let(:attribute) { :respondent_id }
      before { response.respondent = respondent }

      it 'returns the id of the respondent the response belongs to' do
        expect(subject).to be_present
        expect(subject).to eq respondent.id
      end

      it 'returns nil if the response is anonymous' do
        response.anonymous = true
        expect(subject).to eq nil
      end
    end

    describe '#respondent_label' do
      let(:attribute) { :respondent_label }
      before { response.respondent = respondent }

      it 'returns the label of the respondent the response belongs to' do
        expect(subject).to eq 'Stiefvader'
      end

      it 'returns a default label if not present' do
        respondent.label = nil
        expect(subject).to eq '1e Ouder'
      end

      it 'returns the label if the response is anonymous' do
        response.anonymous = true
        expect(subject).to eq 'Stiefvader'
      end
    end

    describe '#respondent_type' do
      let(:attribute) { :respondent_type }
      before { response.respondent = respondent }

      it 'returns the type of the respondent the response belongs to' do
        expect(subject).to eq 'parent'
      end

      it 'returns the type if the response is anonymous' do
        response.anonymous = true
        expect(subject).to eq 'parent'
      end
    end

    describe '#started_at' do
      let(:attribute) { :started_at }

      it 'returns the non_response id' do
        response.started_at = Time.local(2018, 11, 4)
        expect(subject).to eq Time.local(2018, 11, 4)
      end
    end

    describe '#variant' do
      let(:attribute) { :variant }
      let(:response) { Models::Response.exclude(questionnaire_key: :questionnaire_variant).first }

      it 'returns the questionnaire_variant' do
        expect(subject).to eq 'all_quests_v'
      end
    end
  end
end
