# frozen_string_literal: true

require 'spec_helper'

OUTPUT_PATH = RSPEC_ROOT_PATH.join('..', 'tmp', 'data_export') # constant, since it's used in a before(:all)

describe Roqua::CsvExport do
  let(:version_dir) { OUTPUT_PATH.join("example_#{export_version_key}") }
  let(:version_questionnaire_file) { version_dir.join("#{version_questionnaire_key}_#{Time.now.strftime("%Y-%m-%d")}.csv") }
  let(:csv) { CSV.read(version_questionnaire_file, col_sep: ';', headers: true, encoding: 'UTF-8') }
  let(:version_questionnaire_key) { 'all_quests' }
  let(:export_version_key) { 'v201809' }

  before(:all) do
    `rm -rf OUTPUT_PATH`
    Roqua::CsvExport.generate \
      sqlite_path: SQLITE_PATH,
      export_versions_path: RSPEC_ROOT_PATH.join('fixtures', 'export_versions'),
      organization_key: 'example',
      output_path: OUTPUT_PATH
  end

  before(:each) do
    expect(File).to exist version_questionnaire_file
  end

  it 'should be ordered by answer id' do
    answer_ids = csv.map { |row| row['all_quests_id'] }
    expect(answer_ids).to eq answer_ids.sort
  end

  it 'should list first meta_data unprefixed' do
    expect(csv.first['patient_id']).to eq 'doss_epd_id'
    expect(csv.first['respondent_id']).to eq '1'
    expect(csv.first['respondent_type']).to eq 'patient'
    expect(csv.first['respondent_label']).to eq 'Patiënt'
  end

  it 'should list some meta_data prefixed' do
    expect(csv.first['all_quests_protocol']).to eq 'Protocol 1'
    expect(csv.first['all_quests_measurement']).to eq 'Measurement 1'
    expect(csv.first['all_quests_location']).to eq 'Team 1'
  end

  it 'should list questions under their prefixed key' do
    expect(csv.first['all_quests_10']).to eq '3'
  end

  it 'should list scores under their prefixed key' do
    expect(csv.first['all_quests_tot']).to eq '42'
  end

  it 'should list flags and textvars under their prefixed key' do
    expect(csv.first['all_quests_hides_v_1_flag']).to eq 'true'
    expect(csv.first['all_quests_textvar1']).to eq 'var 1'
  end

  it 'should format date values' do
    expect(csv.first['all_quests_open_from']).to eq '02-09-2018 10:00:00'
    expect(csv.first['all_quests_completed_at']).to eq '02-09-2018 11:00:00'
    expect(csv.first['all_quests_date']).to eq '02-09-2018 11:00:00'
  end

  it 'should replace newline characters with spaces (\r\n becomes to for backward compatibility)' do
    expect(csv.first['all_quests_2']).to eq 'long answer 2   with    newlines' # long answer 2 \n with \r\n newlines
    expect(csv.first['all_quests_notes']).to eq 'Note 1   good' # Note 1 \n good
  end

  context 'questions were added after this version' do
    let(:export_version_key) { 'v201803' }

    it "should not have the missing keys in there" do
      expect(csv.first['all_quests_10']).to eq nil
    end
  end

  context 'questionnaire not in export version' do
    let(:version_questionnaire_key) { 'old_all_quests' }
    let(:export_version_key) { 'v201801' }

    it "should save as the first version with the questionnaire" do
      expect(csv.first['old_all_quests_1_oudst']).to eq 'answer 1'
    end
  end

  context 'questionnaire was renamed after the version' do
    let(:version_questionnaire_key) { 'old_all_quests' }
    let(:export_version_key) { 'v201802' }

    it "should save the file prepended with the old key" do
      expect(csv.first['old_all_quests_1_oudst']).to eq 'answer 1'
   end
  end

  context 'question was renamed after the version' do
    let(:export_version_key) { 'v201803' }

    it "should save the question under the old key" do
      expect(csv.first['all_quests_1_oud']).to eq 'answer 1'
    end
  end
end
